@@ -237,7 +237,7 @@ typedef union YYSTYPE
       struct type *type;
     } typed_val_int;
     struct {
-      DOUBLEST dval;
+      gdb_byte val[16];
       struct type *type;
     } typed_val_float;
     struct stoken sval;
@@ -269,12 +269,10 @@ typedef union YYSTYPE
 /* YYSTYPE gets defined by %union.  */
 static int parse_number (struct parser_state *,
 			 const char *, int, int, YYSTYPE *);
-static int parse_go_float (struct gdbarch *gdbarch, const char *p, int len,
-			   DOUBLEST *d, struct type **t);
 
 
 /* Line 264 of yacc.c  */
-#line 278 "go-exp.c"
+#line 276 "go-exp.c"
 
 #ifdef short
 # undef short
@@ -586,13 +584,13 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   193,   193,   194,   197,   204,   205,   210,   214,   218,
-     222,   226,   230,   234,   238,   244,   250,   257,   267,   274,
-     271,   282,   286,   289,   293,   297,   301,   307,   313,   319,
-     323,   327,   331,   335,   339,   343,   347,   351,   355,   359,
-     363,   367,   371,   375,   379,   383,   387,   391,   395,   399,
-     403,   409,   416,   425,   438,   445,   448,   454,   469,   476,
-     493,   511,   523,   529,   535,   551,   603,   605,   612,   625
+       0,   191,   191,   192,   195,   202,   203,   208,   212,   216,
+     220,   224,   228,   232,   236,   242,   248,   255,   265,   272,
+     269,   280,   284,   287,   291,   295,   299,   305,   311,   317,
+     321,   325,   329,   333,   337,   341,   345,   349,   353,   357,
+     361,   365,   369,   373,   377,   381,   385,   389,   393,   397,
+     401,   407,   414,   423,   436,   443,   446,   452,   467,   474,
+     491,   509,   521,   527,   533,   549,   601,   603,   610,   623
 };
 #endif
 
@@ -1643,7 +1641,7 @@ yyreduce:
         case 4:
 
 /* Line 1455 of yacc.c  */
-#line 198 "go-exp.y"
+#line 196 "go-exp.y"
     { write_exp_elt_opcode (pstate, OP_TYPE);
 			  write_exp_elt_type (pstate, (yyvsp[(1) - (1)].tval));
 			  write_exp_elt_opcode (pstate, OP_TYPE); }
@@ -1652,70 +1650,70 @@ yyreduce:
   case 6:
 
 /* Line 1455 of yacc.c  */
-#line 206 "go-exp.y"
+#line 204 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_COMMA); }
     break;
 
   case 7:
 
 /* Line 1455 of yacc.c  */
-#line 211 "go-exp.y"
+#line 209 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_IND); }
     break;
 
   case 8:
 
 /* Line 1455 of yacc.c  */
-#line 215 "go-exp.y"
+#line 213 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_ADDR); }
     break;
 
   case 9:
 
 /* Line 1455 of yacc.c  */
-#line 219 "go-exp.y"
+#line 217 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_NEG); }
     break;
 
   case 10:
 
 /* Line 1455 of yacc.c  */
-#line 223 "go-exp.y"
+#line 221 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_PLUS); }
     break;
 
   case 11:
 
 /* Line 1455 of yacc.c  */
-#line 227 "go-exp.y"
+#line 225 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_LOGICAL_NOT); }
     break;
 
   case 12:
 
 /* Line 1455 of yacc.c  */
-#line 231 "go-exp.y"
+#line 229 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_COMPLEMENT); }
     break;
 
   case 13:
 
 /* Line 1455 of yacc.c  */
-#line 235 "go-exp.y"
+#line 233 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_POSTINCREMENT); }
     break;
 
   case 14:
 
 /* Line 1455 of yacc.c  */
-#line 239 "go-exp.y"
+#line 237 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_POSTDECREMENT); }
     break;
 
   case 15:
 
 /* Line 1455 of yacc.c  */
-#line 245 "go-exp.y"
+#line 243 "go-exp.y"
     { write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
 			  write_exp_string (pstate, (yyvsp[(3) - (3)].ssym).stoken);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT); }
@@ -1724,7 +1722,7 @@ yyreduce:
   case 16:
 
 /* Line 1455 of yacc.c  */
-#line 251 "go-exp.y"
+#line 249 "go-exp.y"
     { mark_struct_expression (pstate);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
 			  write_exp_string (pstate, (yyvsp[(3) - (4)].ssym).stoken);
@@ -1734,7 +1732,7 @@ yyreduce:
   case 17:
 
 /* Line 1455 of yacc.c  */
-#line 258 "go-exp.y"
+#line 256 "go-exp.y"
     { struct stoken s;
 			  mark_struct_expression (pstate);
 			  write_exp_elt_opcode (pstate, STRUCTOP_STRUCT);
@@ -1747,21 +1745,21 @@ yyreduce:
   case 18:
 
 /* Line 1455 of yacc.c  */
-#line 268 "go-exp.y"
+#line 266 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_SUBSCRIPT); }
     break;
 
   case 19:
 
 /* Line 1455 of yacc.c  */
-#line 274 "go-exp.y"
+#line 272 "go-exp.y"
     { start_arglist (); }
     break;
 
   case 20:
 
 /* Line 1455 of yacc.c  */
-#line 276 "go-exp.y"
+#line 274 "go-exp.y"
     { write_exp_elt_opcode (pstate, OP_FUNCALL);
 			  write_exp_elt_longcst (pstate,
 						 (LONGEST) end_arglist ());
@@ -1771,35 +1769,35 @@ yyreduce:
   case 21:
 
 /* Line 1455 of yacc.c  */
-#line 283 "go-exp.y"
+#line 281 "go-exp.y"
     { start_arglist (); }
     break;
 
   case 23:
 
 /* Line 1455 of yacc.c  */
-#line 290 "go-exp.y"
+#line 288 "go-exp.y"
     { arglist_len = 1; }
     break;
 
   case 24:
 
 /* Line 1455 of yacc.c  */
-#line 294 "go-exp.y"
+#line 292 "go-exp.y"
     { arglist_len++; }
     break;
 
   case 25:
 
 /* Line 1455 of yacc.c  */
-#line 298 "go-exp.y"
+#line 296 "go-exp.y"
     { (yyval.lval) = end_arglist () - 1; }
     break;
 
   case 26:
 
 /* Line 1455 of yacc.c  */
-#line 302 "go-exp.y"
+#line 300 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_MEMVAL);
 			  write_exp_elt_type (pstate, (yyvsp[(2) - (4)].tval));
 			  write_exp_elt_opcode (pstate, UNOP_MEMVAL); }
@@ -1808,7 +1806,7 @@ yyreduce:
   case 27:
 
 /* Line 1455 of yacc.c  */
-#line 308 "go-exp.y"
+#line 306 "go-exp.y"
     { write_exp_elt_opcode (pstate, UNOP_CAST);
 			  write_exp_elt_type (pstate, (yyvsp[(1) - (4)].tval));
 			  write_exp_elt_opcode (pstate, UNOP_CAST); }
@@ -1817,161 +1815,161 @@ yyreduce:
   case 28:
 
 /* Line 1455 of yacc.c  */
-#line 314 "go-exp.y"
+#line 312 "go-exp.y"
     { }
     break;
 
   case 29:
 
 /* Line 1455 of yacc.c  */
-#line 320 "go-exp.y"
+#line 318 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_REPEAT); }
     break;
 
   case 30:
 
 /* Line 1455 of yacc.c  */
-#line 324 "go-exp.y"
+#line 322 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_MUL); }
     break;
 
   case 31:
 
 /* Line 1455 of yacc.c  */
-#line 328 "go-exp.y"
+#line 326 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_DIV); }
     break;
 
   case 32:
 
 /* Line 1455 of yacc.c  */
-#line 332 "go-exp.y"
+#line 330 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_REM); }
     break;
 
   case 33:
 
 /* Line 1455 of yacc.c  */
-#line 336 "go-exp.y"
+#line 334 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_ADD); }
     break;
 
   case 34:
 
 /* Line 1455 of yacc.c  */
-#line 340 "go-exp.y"
+#line 338 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_SUB); }
     break;
 
   case 35:
 
 /* Line 1455 of yacc.c  */
-#line 344 "go-exp.y"
+#line 342 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_LSH); }
     break;
 
   case 36:
 
 /* Line 1455 of yacc.c  */
-#line 348 "go-exp.y"
+#line 346 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_RSH); }
     break;
 
   case 37:
 
 /* Line 1455 of yacc.c  */
-#line 352 "go-exp.y"
+#line 350 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_EQUAL); }
     break;
 
   case 38:
 
 /* Line 1455 of yacc.c  */
-#line 356 "go-exp.y"
+#line 354 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_NOTEQUAL); }
     break;
 
   case 39:
 
 /* Line 1455 of yacc.c  */
-#line 360 "go-exp.y"
+#line 358 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_LEQ); }
     break;
 
   case 40:
 
 /* Line 1455 of yacc.c  */
-#line 364 "go-exp.y"
+#line 362 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_GEQ); }
     break;
 
   case 41:
 
 /* Line 1455 of yacc.c  */
-#line 368 "go-exp.y"
+#line 366 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_LESS); }
     break;
 
   case 42:
 
 /* Line 1455 of yacc.c  */
-#line 372 "go-exp.y"
+#line 370 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_GTR); }
     break;
 
   case 43:
 
 /* Line 1455 of yacc.c  */
-#line 376 "go-exp.y"
+#line 374 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_AND); }
     break;
 
   case 44:
 
 /* Line 1455 of yacc.c  */
-#line 380 "go-exp.y"
+#line 378 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_XOR); }
     break;
 
   case 45:
 
 /* Line 1455 of yacc.c  */
-#line 384 "go-exp.y"
+#line 382 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_BITWISE_IOR); }
     break;
 
   case 46:
 
 /* Line 1455 of yacc.c  */
-#line 388 "go-exp.y"
+#line 386 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_LOGICAL_AND); }
     break;
 
   case 47:
 
 /* Line 1455 of yacc.c  */
-#line 392 "go-exp.y"
+#line 390 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_LOGICAL_OR); }
     break;
 
   case 48:
 
 /* Line 1455 of yacc.c  */
-#line 396 "go-exp.y"
+#line 394 "go-exp.y"
     { write_exp_elt_opcode (pstate, TERNOP_COND); }
     break;
 
   case 49:
 
 /* Line 1455 of yacc.c  */
-#line 400 "go-exp.y"
+#line 398 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_ASSIGN); }
     break;
 
   case 50:
 
 /* Line 1455 of yacc.c  */
-#line 404 "go-exp.y"
+#line 402 "go-exp.y"
     { write_exp_elt_opcode (pstate, BINOP_ASSIGN_MODIFY);
 			  write_exp_elt_opcode (pstate, (yyvsp[(2) - (3)].opcode));
 			  write_exp_elt_opcode (pstate, BINOP_ASSIGN_MODIFY); }
@@ -1980,7 +1978,7 @@ yyreduce:
   case 51:
 
 /* Line 1455 of yacc.c  */
-#line 410 "go-exp.y"
+#line 408 "go-exp.y"
     { write_exp_elt_opcode (pstate, OP_LONG);
 			  write_exp_elt_type (pstate, (yyvsp[(1) - (1)].typed_val_int).type);
 			  write_exp_elt_longcst (pstate, (LONGEST)((yyvsp[(1) - (1)].typed_val_int).val));
@@ -1990,7 +1988,7 @@ yyreduce:
   case 52:
 
 /* Line 1455 of yacc.c  */
-#line 417 "go-exp.y"
+#line 415 "go-exp.y"
     {
 			  struct stoken_vector vec;
 			  vec.len = 1;
@@ -2002,7 +2000,7 @@ yyreduce:
   case 53:
 
 /* Line 1455 of yacc.c  */
-#line 426 "go-exp.y"
+#line 424 "go-exp.y"
     { YYSTYPE val;
 			  parse_number (pstate, (yyvsp[(1) - (1)].ssym).stoken.ptr,
 					(yyvsp[(1) - (1)].ssym).stoken.length, 0, &val);
@@ -2017,17 +2015,17 @@ yyreduce:
   case 54:
 
 /* Line 1455 of yacc.c  */
-#line 439 "go-exp.y"
-    { write_exp_elt_opcode (pstate, OP_DOUBLE);
+#line 437 "go-exp.y"
+    { write_exp_elt_opcode (pstate, OP_FLOAT);
 			  write_exp_elt_type (pstate, (yyvsp[(1) - (1)].typed_val_float).type);
-			  write_exp_elt_dblcst (pstate, (yyvsp[(1) - (1)].typed_val_float).dval);
-			  write_exp_elt_opcode (pstate, OP_DOUBLE); }
+			  write_exp_elt_floatcst (pstate, (yyvsp[(1) - (1)].typed_val_float).val);
+			  write_exp_elt_opcode (pstate, OP_FLOAT); }
     break;
 
   case 56:
 
 /* Line 1455 of yacc.c  */
-#line 449 "go-exp.y"
+#line 447 "go-exp.y"
     {
 			  write_dollar_variable (pstate, (yyvsp[(1) - (1)].sval));
 			}
@@ -2036,7 +2034,7 @@ yyreduce:
   case 57:
 
 /* Line 1455 of yacc.c  */
-#line 455 "go-exp.y"
+#line 453 "go-exp.y"
     {
 			  /* TODO(dje): Go objects in structs.  */
 			  write_exp_elt_opcode (pstate, OP_LONG);
@@ -2054,7 +2052,7 @@ yyreduce:
   case 58:
 
 /* Line 1455 of yacc.c  */
-#line 470 "go-exp.y"
+#line 468 "go-exp.y"
     {
 			  /* TODO(dje): Go objects in structs.  */
 			  write_exp_elt_opcode (pstate, UNOP_SIZEOF);
@@ -2064,7 +2062,7 @@ yyreduce:
   case 59:
 
 /* Line 1455 of yacc.c  */
-#line 477 "go-exp.y"
+#line 475 "go-exp.y"
     {
 			  /* We copy the string here, and not in the
 			     lexer, to guarantee that we do not leak a
@@ -2085,7 +2083,7 @@ yyreduce:
   case 60:
 
 /* Line 1455 of yacc.c  */
-#line 494 "go-exp.y"
+#line 492 "go-exp.y"
     {
 			  /* Note that we NUL-terminate here, but just
 			     for convenience.  */
@@ -2106,7 +2104,7 @@ yyreduce:
   case 61:
 
 /* Line 1455 of yacc.c  */
-#line 512 "go-exp.y"
+#line 510 "go-exp.y"
     {
 			  int i;
 
@@ -2121,7 +2119,7 @@ yyreduce:
   case 62:
 
 /* Line 1455 of yacc.c  */
-#line 524 "go-exp.y"
+#line 522 "go-exp.y"
     { write_exp_elt_opcode (pstate, OP_BOOL);
 			  write_exp_elt_longcst (pstate, (LONGEST) (yyvsp[(1) - (1)].lval));
 			  write_exp_elt_opcode (pstate, OP_BOOL); }
@@ -2130,7 +2128,7 @@ yyreduce:
   case 63:
 
 /* Line 1455 of yacc.c  */
-#line 530 "go-exp.y"
+#line 528 "go-exp.y"
     { write_exp_elt_opcode (pstate, OP_BOOL);
 			  write_exp_elt_longcst (pstate, (LONGEST) (yyvsp[(1) - (1)].lval));
 			  write_exp_elt_opcode (pstate, OP_BOOL); }
@@ -2139,7 +2137,7 @@ yyreduce:
   case 64:
 
 /* Line 1455 of yacc.c  */
-#line 536 "go-exp.y"
+#line 534 "go-exp.y"
     { struct symbol *sym = (yyvsp[(1) - (2)].ssym).sym.symbol;
 
 			  if (sym == NULL
@@ -2158,7 +2156,7 @@ yyreduce:
   case 65:
 
 /* Line 1455 of yacc.c  */
-#line 552 "go-exp.y"
+#line 550 "go-exp.y"
     { struct block_symbol sym = (yyvsp[(1) - (1)].ssym).sym;
 
 			  if (sym.symbol)
@@ -2205,21 +2203,21 @@ yyreduce:
   case 66:
 
 /* Line 1455 of yacc.c  */
-#line 604 "go-exp.y"
+#line 602 "go-exp.y"
     { (yyval.tval) = lookup_pointer_type ((yyvsp[(2) - (2)].tval)); }
     break;
 
   case 67:
 
 /* Line 1455 of yacc.c  */
-#line 606 "go-exp.y"
+#line 604 "go-exp.y"
     { (yyval.tval) = (yyvsp[(1) - (1)].tsym).type; }
     break;
 
   case 68:
 
 /* Line 1455 of yacc.c  */
-#line 613 "go-exp.y"
+#line 611 "go-exp.y"
     { (yyval.tval) = builtin_go_type (parse_gdbarch (pstate))
 			    ->builtin_uint8; }
     break;
@@ -2227,7 +2225,7 @@ yyreduce:
 
 
 /* Line 1455 of yacc.c  */
-#line 2232 "go-exp.c"
+#line 2230 "go-exp.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2439,27 +2437,9 @@ yyreturn:
 
 
 /* Line 1675 of yacc.c  */
-#line 635 "go-exp.y"
+#line 633 "go-exp.y"
 
 
-/* Wrapper on parse_c_float to get the type right for Go.  */
-
-static int
-parse_go_float (struct gdbarch *gdbarch, const char *p, int len,
-		DOUBLEST *d, struct type **t)
-{
-  int result = parse_c_float (gdbarch, p, len, d, t);
-  const struct builtin_type *builtin_types = builtin_type (gdbarch);
-  const struct builtin_go_type *builtin_go_types = builtin_go_type (gdbarch);
-
-  if (*t == builtin_types->builtin_float)
-    *t = builtin_go_types->builtin_float32;
-  else if (*t == builtin_types->builtin_double)
-    *t = builtin_go_types->builtin_float64;
-
-  return result;
-}
-
 /* Take care of parsing a number (anything that starts with a digit).
    Set yylval and return the token type; update lexptr.
    LEN is the number of characters in it.  */
@@ -2496,10 +2476,34 @@ parse_number (struct parser_state *par_state,
 
   if (parsed_float)
     {
-      if (! parse_go_float (parse_gdbarch (par_state), p, len,
-			    &putithere->typed_val_float.dval,
-			    &putithere->typed_val_float.type))
-	return ERROR;
+      const struct builtin_go_type *builtin_go_types
+	= builtin_go_type (parse_gdbarch (par_state));
+
+      /* Handle suffixes: 'f' for float32, 'l' for long double.
+	 FIXME: This appears to be an extension -- do we want this?  */
+      if (len >= 1 && tolower (p[len - 1]) == 'f')
+	{
+	  putithere->typed_val_float.type
+	    = builtin_go_types->builtin_float32;
+	  len--;
+	}
+      else if (len >= 1 && tolower (p[len - 1]) == 'l')
+	{
+	  putithere->typed_val_float.type
+	    = parse_type (par_state)->builtin_long_double;
+	  len--;
+	}
+      /* Default type for floating-point literals is float64.  */
+      else
+        {
+	  putithere->typed_val_float.type
+	    = builtin_go_types->builtin_float64;
+        }
+
+      if (!parse_float (p, len,
+			putithere->typed_val_float.type,
+			putithere->typed_val_float.val))
+        return ERROR;
       return FLOAT;
     }
 
@@ -3105,7 +3109,7 @@ static int popping;
 
 /* Temporary storage for yylex; this holds symbol names as they are
    built up.  */
-static struct obstack name_obstack;
+static auto_obstack name_obstack;
 
 /* Build "package.name" in name_obstack.
    For convenience of the caller, the name is NUL-terminated,
@@ -3117,7 +3121,7 @@ build_packaged_name (const char *package, int package_len,
 {
   struct stoken result;
 
-  obstack_free (&name_obstack, obstack_base (&name_obstack));
+  name_obstack.clear ();
   obstack_grow (&name_obstack, package, package_len);
   obstack_grow_str (&name_obstack, ".");
   obstack_grow (&name_obstack, name, name_len);
@@ -3368,18 +3372,13 @@ yylex (void)
 int
 go_parse (struct parser_state *par_state)
 {
-  int result;
-  struct cleanup *back_to;
-
   /* Setting up the parser state.  */
+  scoped_restore pstate_restore = make_scoped_restore (&pstate);
   gdb_assert (par_state != NULL);
   pstate = par_state;
 
-  back_to = make_cleanup (null_cleanup, NULL);
-
   scoped_restore restore_yydebug = make_scoped_restore (&yydebug,
 							parser_debug);
-  make_cleanup_clear_parser_state (&pstate);
 
   /* Initialize some state used by the lexer.  */
   last_was_structop = 0;
@@ -3387,12 +3386,9 @@ go_parse (struct parser_state *par_state)
 
   VEC_free (token_and_value, token_fifo);
   popping = 0;
-  obstack_init (&name_obstack);
-  make_cleanup_obstack_free (&name_obstack);
+  name_obstack.clear ();
 
-  result = yyparse ();
-  do_cleanups (back_to);
-  return result;
+  return yyparse ();
 }
 
 void
